;
;	DDS_628A_QBH.ASM 
;	32.768 KHz TMR1 time base
;	Internal 4 MHz clk, Ext /MCLR
;	Transmits callsign as 5 Hz FSK QRSS Morse
;	6 sec/bit Stackable 10 min timing
;	Tuning and call sign entry with rotary encoder
;	
;	Morse code engine v2
;	16x2 LCD display - Backlight optional
;	10.140 MHz CAL freq with offset stored in EE mem
;	Encoder SW is A:4
;	Pushbutton SW is A:6
;	Mode SW is A:7
;	
;	Default freq is 14,025,00 KHz	
;	
; 	7/30/11 	
;	
;******************************************************************************
;
; 	Modification History
; 	Original Author - Curtis W. Preuss - WB2V
;    	8/19/98 - Version 1 - Initial Version by Curtis W. Preuss - WB2V
;   	12/xx/98 - Version 2 - Converted to MPASM by Bruce Stough, AA0ED 
;   	4/21/99 - Version 3 - Fixed and modified by 
;                            Bruce Stough, AA0ED (sbs1@visi.com) and
;                            Craig Johnson, AA0ZZ (aa0zz@arrl.net)
;
; 	6/5/11 - Modified for PIC16F628A and added QRSS transmit by Mike Berg N0QBH
;
;*****************************************************************************
;
; Target Controller -      PIC16F628A
;                          __________
;     LCD_rs----------RA2 |1       18| RA1---------ENCODER A
;     LCD_rw----------RA3 |2       17| RA0---------ENCODER B
;     ENCODER SWITCH--RA4 |3       16| RA7---------QR SWITCH
;     /MCLR-----------RA5 |4       15| RA6---------PB SWITCH
;     Ground----------Vss |5       14| VDD---------+5 V
;     LCD 11----------RB0 |6       13| RB7---------XTAL 32 KHz
;     DDS CLK/LCD 12--RB1 |7       12| RB6---------XTAL
;     LCD 13----------RB2 |8       11| RB5---------LCD_e
;     DDS DATA/LCD 14-RB3 |9       10| RB4---------DDS LOAD
;                          ----------
;	
; ****************************************************************************
; * Device type and options.                                                 *
; ****************************************************************************
;
        LIST    P=16F628A
	ERRORLEVEL	-302
        include <P16F628A.INC>
        __CONFIG        h'3f30'
;
;------------------------------------------------------------
;
	ORG	0			; start code
	goto	start
	ORG	4
	goto	interrupt_service	;interrupt vector
;
; ****************************************************************************
; *           Allocate variables in general purpose register space           *
; ****************************************************************************
;
        cblock  0x20
        temp_S                  	; Interrupt temp store
        temp_W                  	;
        freq_0                    	; Display frequency (hex) 
          freq_1                  	;  (4 bytes) 
          freq_2
          freq_3         
        BCD_0                     	; Display frequency (BCD) 
          BCD_1                  	 ;  (5 bytes)
          BCD_2
          BCD_3
          BCD_4     
        AD9851_0                  	; AD9851 control word 
          AD9851_1                	;  (5 bytes)
          AD9851_2
          AD9851_3
          AD9851_4
        fstep_0                   	;(30h) Frequency inc/dec 
          fstep_1                 	;  (4 bytes)
          fstep_2			;
          fstep_3
        BCD_count                 	; Used in bin2BCD routine
        BCD_temp                  	;   "
        mult_count                	; Used in calc_dds_word 
        bit_count                 	;   "
        byte2send                 	;
        osc_0                     	; Current oscillator 
          osc_1                   	;  (4 bytes)
          osc_2
          osc_3
        osc_temp_0                	; Oscillator frequency 
          osc_temp_1              	;  (4 bytes)
          osc_temp_2        
          osc_temp_3			; (40h)
        LCD_char                  	; Character being sent to the LCD
        LCD_read                  	; Character read from the LCD
        timer1                    	; Used in delay routines
        timer2                    	;   "
        ren_timer_0               	; For variable rate tuning 
          ren_timer_1             	;  (2 bytes)
        ren_new                   	; New value of encoder pins A and B
        ren_old                   	; Old value of encoder pins A and B
        ren_read                  	; PORTA inputs
        last_dir                  	; Indicates last direction of encoder
        next_dir                  	; Indicates expected direction
        count                     	; loop counter  (gets reused)
        band                      	; Used to index a table of frequencies
        rs_value                  	; The LCD rs line flag value
;
	stacker_l			;
	stacker_h			; (50h)
	six_second			; qrss timing
	roll_cnt			; bit pointer
	morse_bytes			; total bytes
	buffer_2			; bitwise rotating buffer
	pointer				; points
	out_count			; bits leaving
	count1				; 25h util (b_b, csr only)
	pass_cnt			; 2 loops (b_b, csr only)
	morse_bits			; total bits (b_b only)
	buffer_1			; ascii in buffer (b_b only)
	rm_buffer			; yet another (b_b only)
	pointer1			; pointer
;
	ee_addr				; EE memory regs
	ee_data				;
	FLAGS				; (5Fh)
;
        endc                      	; End of Data Block
;
;                  
;    Note:   2^32 = 4294967296 and 2^24 = 16777216
;
; For example, for a 180 MHz clock:
;    ref_osc_3 is (2^32 / 120 x 10^6) = 23.86092942 truncated to 23 (0x17h)  
;    ref_osc_2 is the high byte of (.86092942 x 2^24) = 14443998.88         
;    14443998.88 = 0xDC65DE hex, so high byte is DC.                           
;    ref_osc_1 is the next byte of 0xDC65DE, or 65
;    ref_osc_0 is the last byte of 0xDC65DE, or DE
;
;==== Currently set for 180 MHz Oscillator =======
;
ref_osc_3   	equ 	0x17		; Most significant osc byte
ref_osc_2   	equ 	0xDC            ; Next byte
ref_osc_1   	equ 	0x65            ; Next byte
ref_osc_0   	equ 	0xDE            ; Least significant byte
;
; Limit contains the upper limit frequency as a 32 bit integer.
;
limit_3  	equ 	0x01            ; Most significant byte for 30 MHz
limit_2   	equ 	0xC9            ; Next byte
limit_1   	equ 	0xC3            ; Next byte
limit_0   	equ 	0x80            ; Least significant byte
;
; Default contains the default startup frequency as a 32 bit integer.
;
default_3 	equ 	0x00            ; Most significant byte for 14.025 MHz
default_2 	equ 	0xD6            ; Next byte
default_1 	equ 	0x01            ; Next byte
default_0 	equ 	0x28            ; Least significant byte 
;
band_end 	equ    	0x28            ; The offset to the last band table entry 
RAM_START	equ	0x60		; bin store start
TENMIN		equ	.300		; 10 minutes
LINE1		equ	0x80		; start of LCD line #1
LINE2		equ	0xc0		; "		"  #2
;
; ****************************************************************************
; * Assign names to IO pins.                                                 *
; ****************************************************************************
;
#define	LCD_rs		PORTA,2		; 0=instruction, 1=data
#define	LCD_rw  	PORTA,3    	; 0=write, 1=read
#define	KB_SW		PORTA,4		; encoder PB sw 
#define	PB_SW 		PORTA,6         ; Calibrate Push Button, (active low)
#define	QR_SW 		PORTA,7         ; Select VFO(hi) / QRSS(lo) modes
#define	DDS_load 	PORTB,4    	; Update pin on AD9851 (active high)
#define	LCD_e   	PORTB,5    	; 0=disable, 1=enable
; 
#define	MFLAG		FLAGS,0		; toggle morse
#define CS_FLAG		FLAGS,1		; set after call sign has been set
;
; ****************************************************************************  
; * This is the band table.  Each entry is four instructions long, with each *
; * group of four literals representing the frequency as a 32 bit integer.   *
; ****************************************************************************
;
band_table
        addwf   PCL,f             ; 
	dt	0x00,0x00,0x00,0x00	; 0 Hz
	dt	0x00,0x1B,0x77,0x40	; 160 meters
        dt	0x00,0x35,0x67,0xE0     ; 80 meters
	dt 	0x00,0x6A,0xCF,0xC0	; 40 meters
	dt	0x00,0x9A,0x1D,0x20	; 30 meters
	dt	0x00,0xD5,0x9F,0x80	; 20 meters
 	dt	0x01,0x13,0xB2,0x20	; 17 meters
 	dt	0x01,0x40,0x6F,0x40	; 15 meters
	dt	0x01,0x7B,0xCA,0x90	; 12 meters
 	dt	0x01,0xAB,0x3F,0x00	; 10 meters
	dt	0x01,0xC9,0xC3,0x80	; 30 MHz
;
qrss_lu_txt
        addwf   PCL,f             	; 
	dt	"tx 6s",0		;	
;
; *****************************************************************************
; *                                                                           *
; *****************************************************************************
;
start
        movlw   0x07
        movwf   CMCON			; analog inputs off
        bsf     STATUS,RP0              ; switch to bank 1
        movlw   b'11010011'             ; port A I/O
        movwf   TRISA
        movlw   b'11000000'             ; port B I/O
        movwf   TRISB
        movlw   b'10000001'             ; WPU off, TMR0/4
        movwf   OPTION_REG		;
        movlw   b'00000001'             ; TMR1
        movwf   PIE1                    ; perif int setup
;
        bcf     STATUS,RP0              ; back to page 0
;
        clrf    PIR1                    ; reset perif int
        movlw   b'00000000'             ; Ints off
	movwf	INTCON
        movlw   b'00001010'		; 
        movwf   T1CON                   ; TMR1/1, ext osc, off
	movlw	b'00000000'		;
	movwf	T2CON			; TMR2 post/1, off 
	clrf 	FLAGS			;
;
        call    init_LCD          	; Initialize the LCD
;
	btfsc	KB_SW			; test enc pb switch at pwr up
	goto	bb_zone			; high, normal mode
	call	cs_rodeo		; user inputs call w/enc
; 
	bsf	CS_FLAG			; set flag
	call	line1_clearer		; erase LCD line 1
	call	line2_clearer		; erase LCD line 2
;
bb_zone
	call	bin_builder		; write morse
	movlw	high bb_zone
	movwf	PCLATH			; restore pclath
;
	movlw	RAM_START -1		; this is done once
	subwf	pointer,W		;
	movwf	morse_bytes		;
;	
;       Enter Calibrate Mode if push button is pressed while turning the 
;       power on. Coming out of call set ignores calibrate.
;
        btfsc   PB_SW   		; Is the switch pressed?
        goto    read_EEocs        	; No, get clock freq from EEPROM
	btfss	CS_FLAG			; Yes, call flag set?	
        call    calibrate         	; no, calibrate
	bcf	CS_FLAG			; clear flag
;
;       Get the reference oscillator constant from the EEPROM.
;
read_EEocs
        clrf    ee_addr             	; Reset the EEPROM read address
        call    ee_read       		; Read EEPROM
        movf    ee_data,w          	; Get the first osc byte
        movwf   osc_0             	; Save osc frequency
	incf	ee_addr,f		;
        call    ee_read       		; Get next byte
        movf    ee_data,w          	; 
        movwf   osc_1             	; Save it
	incf	ee_addr,f		;
        call    ee_read       		; Get the third byte
        movf    ee_data,w          	; 
        movwf   osc_2             	; Save it
	incf	ee_addr,f		;
        call    ee_read       		; Get the fourth byte
        movf    ee_data,w          	;
        movwf   osc_3             	; Save it
;
;       Set the power on frequency to the defined value.
;
        movlw   default_0         	; Get the least significant byte
        movwf   freq_0            	; Save it
        movlw   default_1         	; Get the next byte
        movwf   freq_1            	; Save it
        movlw   default_2         	; And the next
        movwf   freq_2            	; Save it
        movlw   default_3         	; Get the most significant byte
        movwf   freq_3            	; Save it
;
;       Display the power on frequency.
;

        call    bin2BCD           	; Convert it to BCD
        call    show_freq         	; Display it
;
;       Send power on frequency to the DDS chip.
;
        call    calc_dds_word     	; Convert to delta value
        call    send_dds_word     	; Send the power-on frequency to the
                                  	; AD9851 in serial mode
	call	wait_128ms
        call    send_dds_word     	; resend (test)
;
;       Get the power on encoder value.
;
reentryA
        movf    PORTA,w           	; Read port A
        movwf   ren_read          	; Save it in ren_read
        movlw   0x03              	; Get encoder mask
        andwf   ren_read,w        	; Get encoder bits
        movwf   ren_old           	; Save in ren_old
;
;       Initialize variables.
;
        clrf    ren_timer_1       	; Initialize the encoder speed timer        
        movlw   0x40              	; to                                     
        movwf   ren_timer_0       	; 0x0040                               
        clrf    last_dir          	; Clear the knob direction indicator
        clrf    band              	; Clear the band indicator
; 
; 	Fall into the Main Program Loop
;
main
        call    poll_encoder      	; Check for knob movement
;
	movfw	PORTA
	movwf	ren_read		;
        movlw   b'10000000'    		; Get encoder mask
        andwf   ren_read,w        	; Get encoder bits
	bnz	bander			; bit 7 low?
	call	wait_64ms		; yes, debounce
	movfw	PORTA
	movwf	ren_read		;
        movlw   b'10000000'    		; mask
        andwf   ren_read,w        	; Get encoder bit
	bz	qrss_land		; debounced, QR_SW closed
bander
        btfss   ren_read,6        	; Change band? (button pushed?)
        goto    change_band       	; Yes, change band
;
;       Determine step size to use (1 Hz or 1 kHz).
;
        clrf    fstep_3           	; Guess that we want 1 Hz steps by 
        clrf    fstep_2           	; setting fstep to one.
        clrf    fstep_1           	; 
        movlw   0x01              	;
        movwf   fstep_0           	; 
        btfsc   ren_read,4        	; Is the encoder switch pressed?
        goto    go_step           	; No, use the 1 Hz step
        movlw   0xE8              	; Yes, set the step value to 1 kHz
        movwf   fstep_0           	; by setting fstep_0 to 0xE8 and
        movlw   0x03              	; fstep_1 to 0x03
        movwf   fstep_1           	; 
        goto    go_step           	; Use the 1 kHz step
;
;       Adjust the tuning step based on ren_timer.
;
bump_step
        bcf     STATUS,C        	; Clear the carry flag
        rlf     fstep_0,f         	; Multiply the step by 2 by rotating left
        rlf     fstep_1,f         	; 
        rlf     fstep_2,f         	; 
        rlf     fstep_3,f         	; 
go_step
        rlf     ren_timer_0,f     	; Multiply the encoder timer by 2
        rlf     ren_timer_1,f     	;
        btfss   STATUS,C        	; Has a one floated to the carry yet?
        goto    bump_step         	; No, then double the step size
;
;       Based on the knob direction, either add or subtract the increment, 
;       then update the LCD and DDS.
; 
        btfsc   last_dir,1        	; Is the knob going up?
        goto    up                	; Yes, then add the increment
down
        call    sub_step          	; Subtract fstep from freq
        goto    write             	; Update LCD and DDS
up
        call    add_step          	; Add fstep to freq
        call    check_add         	; Make sure we did not exceed the maximum
        goto    write             	; Update LCD and DDS
;
;------------------------------------------------------------
;
change_band
        btfsc   last_dir,1        	; Are we going up in the band list? 
        goto    band_up           	; Yes, increment band address
        movlw   0x04              	; No, get 4 bytes to subtract
        subwf   band,f            	; Move down in band list
        movlw   0xFF-band_end     	; Check to see if we have fallen off the
        addwf   band,w            	; bottom of the table.
        btfss   STATUS,C        	; Off the bottom?
        goto    valid             	; No, continue
        movlw   band_end          	; Yes, go to highest entry
        movwf   band              	; 
valid
        call    get_band          	; Get the new band frequency
        goto    write             	; Set the frequency and continue
band_up
        movlw   0x04              	; Table entries are 4 bytes apart
        addwf   band,f            	; Increment the band pointer
        movlw   0xFF-band_end     	; Check to see if we have gone over the
        addwf   band,w            	; top of the table.
        btfsc   STATUS,C        	; Did we go over the top of the table?
        clrf    band              	; Yes, go to the bottom entry
        call    get_band          	; Get the new band frequency
;
write
        call    bin2BCD           	; Convert the frequency to BCD
        call    show_freq         	; Display the frequency on the LCD
        call    calc_dds_word     	; Find the control word for the DDS chip
        call    send_dds_word     	; Send the control word to the DDS chip
        goto    main              	; Continue polling the encoder (endless loop)
;
;------------------------------------------------------------
;
qrss_land
	movlw	LINE2 +1		;
        movwf   LCD_char          	; skip 1st LCD char
        call    cmnd2LCD          	; points to line 2 
	movlw	.8
	movwf	ee_addr			; start of call	
qrss_land_call
	call	ee_read			; read call in eemem
	movfw	ee_data			; ee_data to W
	bz	qrss_txt		; saw zero, exit
        movwf   LCD_char          	; 
        call    data2LCD          	; display byte
	incf	ee_addr,F		; next ee byte
	goto	qrss_land_call		; loop
;
qrss_txt
	movlw	LINE2 +.8		;
        movwf   LCD_char          	; skip 1st LCD char
        call    cmnd2LCD          	; points to line 2 
	clrf	pointer1
qrss_txt_loop
	movfw	pointer1		;
	call	qrss_lu_txt	
        movwf   LCD_char          	; 
	movfw	LCD_char
	bz	qrss_land_call_done
        call    data2LCD          	;
	incf	pointer1,F
	goto	qrss_txt_loop		;
;
qrss_land_call_done
	movlw	0xa0			; start of unshifted frq
	movwf	FSR			; mid mem start
        movfw   freq_0            	; Save it in mid memory
	movwf	INDF
	incf	FSR,F
        movfw   freq_1            	; Save it
	movwf	INDF
	incf	FSR,F
        movfw   freq_2            	; Save it
	movwf	INDF
	incf	FSR,F
        movfw   freq_3            	; Save it
	movwf	INDF
;
        clrf    fstep_3           	; shift freq 5 Hz
        clrf    fstep_2           	; 
        clrf    fstep_1           	; 
        movlw   0x05              	; 5 Hz
        movwf   fstep_0           	;
	call	add_step		; freq_0-3 +5
;
	movlw	0xa4			;
	movwf	FSR			; start of shifted freq
        movfw   freq_0            	; Save it(+5Hz) in mid memory
	movwf	INDF
	incf	FSR,F
        movfw   freq_1            	; Save it
	movwf	INDF
	incf	FSR,F
        movfw   freq_2            	; Save it
	movwf	INDF
	incf	FSR,F
        movfw   freq_3            	; Save it
	movwf	INDF
;
	movlw	.3			; preload 6 sec counter
	movwf	six_second		; for 3 loops
	clrf	TMR1H
	clrf	TMR1L			; reset
	bcf	PIR1,TMR1IF
	bsf	T1CON,TMR1ON		; turn timer on
	movlw	b'11000000'		;
	movwf	INTCON			; turn on T1 int
;
fromthetop
	clrf	stacker_h		; reset
	clrf	stacker_l		;
	movfw	morse_bytes		; preserve count
	movwf	out_count		; num of bytes total
	movlw	RAM_START			;
	movwf	pointer			; start of binary storage
;
bin_made_loop
	movfw	pointer			; point to last 	
	movwf	FSR			;
	movfw	INDF			; get byte pointed by FSR
	movwf	buffer_2		; copy
	movlw	.8
	movwf 	roll_cnt		; reload bit counter
bin_made_inner
	btfsc	QR_SW			; If bit is high
	goto	vfo_setup		; break out of QRSS <<<
	nop
	btfss	MFLAG			; wait for 6 sec timeout
	goto	bin_made_inner		; keep looping
	bcf	MFLAG			;
;
	rlf	buffer_2,F		; rotate copy of byte
	skpnc				;
	goto	tone_high		; carry = 1
tone_low
	movlw	0xa0			; carry = 0
	movwf	FSR
	goto	beyond_sort		; load org freq
tone_high
 	movlw	0xa4
	movwf	FSR			; load shifted freq
beyond_sort
	movfw	INDF			; get stored
	movwf	freq_0			; load freq regs
	incf	FSR,F			; inc FSR
	movfw	INDF			;
	movwf	freq_1			;
	incf	FSR,F			;
	movfw	INDF			;
	movwf	freq_2			;
	incf	FSR,F			;
	movfw	INDF			;
	movwf	freq_3			; all regs loaded
;
;       Send frequency to the DDS chip.
;
        call    calc_dds_word     	; Convert to delta value
        call    send_dds_word     	; Send the frequency
;
;	Display freq
;
        call    bin2BCD           	; Convert it to BCD
        call    show_freq         	; Display it
;
beyond_dds
	decfsz	roll_cnt,F		; count down the bits
	goto	bin_made_inner		;
;
	incf	pointer,F		; point to next byte
	decfsz	out_count,F		; counts down total bytes
	goto	bin_made_loop		; loop back for new byte
;
endofstring
	movlw	0xa0			; org freq is selected
	movwf	FSR			;
	movfw	INDF			;
	movwf	freq_0			;
	incf	FSR,F			;
	movfw	INDF			;
	movwf	freq_1			;
	incf	FSR,F			;
	movfw	INDF			;
	movwf	freq_2			;
	incf	FSR,F			;
	movfw	INDF			;
	movwf	freq_3			;
        call    bin2BCD           	; Convert it to BCD
        call    show_freq         	; Display it
;
;       Send frequency to the DDS chip.
;
        call    calc_dds_word     	; Convert to delta value
        call    send_dds_word     	; Send the frequency
;
waitfor10min
	btfsc	QR_SW			; If bit is low
	goto	vfo_setup		; break out of QRSS <<<
	movlw	high TENMIN		; 
	subwf	stacker_h,W		;
	bnc	waitfor10min		; high TENMIN > stacker_h
	skpz				;
	goto	fromthetop		; high TENMIN < stacker_h 	
	movlw	low TENMIN		; high TENMIN = stacker_h
	subwf	stacker_l,W		; 
	skpc				;
	goto	waitfor10min		; low TENMIN > stacker_l, loop	
	goto	fromthetop		; low TENMIN <= stacker_l
;
vfo_setup
	bcf	INTCON,GIE		; interrupts off
	btfsc	INTCON,GIE		;
	goto	vfo_setup		;
	bcf	T1CON,TMR1ON		; turn timer off
	clrf	TMR1H
	clrf	TMR1L			; reset
	bcf	PIR1,TMR1IF
;
	call	line1_clearer		; erase LCD line 1
	call	line2_clearer		; erase LCD line 2
	movlw	0xa0			; org freq is selected
	movwf	FSR			;
	movfw	INDF			;
	movwf	freq_0			;
	incf	FSR,F			;
	movfw	INDF			;
	movwf	freq_1			;
	incf	FSR,F			;
	movfw	INDF			;
	movwf	freq_2			;
	incf	FSR,F			;
	movfw	INDF			;
	movwf	freq_3			;
        call    bin2BCD           	; Convert it to BCD
        call    show_freq         	; Display it
;
;       Send frequency to the DDS chip.
;
        call    calc_dds_word     	; Convert to delta value
        call    send_dds_word     	; Send the frequency
	
	goto	reentryA		; return to VFO	
;
;============================================================
;
;	32 kHz xtal gives interrupt every 2 seconds
;
interrupt_service
	movwf	temp_W			; store W & Status
	swapf	STATUS,W
	movwf	temp_S
	btfss	PIR1,TMR1IF
	goto	outta_here	
	bcf	PIR1,TMR1IF		; clear flag
;
	incf	stacker_l,F		; stacker_l
	skpnz
	incf	stacker_h,F		; stacker_h
	decfsz	six_second,F		; skip next every 6 sec
	goto	outta_here
;
	movlw	.3			; reload 6 sec counter
	movwf	six_second		; for 3 loops
	bsf	MFLAG			; set flag
;
outta_here
	swapf	temp_S,W		; restore W & Status
	movwf	STATUS
	swapf	temp_W,F
	swapf	temp_W,W
	retfie
;
;------------------------------------------------------------
;
get_band
        movf    band,w            	; Get the index of the high byte 
        call    band_table        	; Get the value into W
        movwf   freq_3            	; Save it in freq_3
        incf    band,f            	; Increment index to next byte
        movf    band,w            	; Get the index of the next byte
        call    band_table        	; Get the value into W
        movwf   freq_2            	; Save it in freq_2
        incf    band,f            	; Increment index to the next byte
        movf    band,w            	; Get the index to the next byte
        call    band_table        	; Get the value into W
        movwf   freq_1            	; Save it in freq_1
        incf    band,f            	; Increment index to the low byte
        movf    band,w            	; Get the index to the low byte
        call    band_table        	; Get the value into W
        movwf   freq_0            	; Save it in freq_0
        movlw   0x03              	; Get a constant three
        subwf   band,f            	; Restore original value of band
        return                    	; Return to the caller
;
init_LCD
        call    wait_64ms         	; Wait for LCD to power up
        movlw   0x03              	; LCD init instruction (First)RB:0-3
        movwf   PORTB             	; Send to LCD via RB3..RB0
        bsf     LCD_e       		; Set the LCD E line high,
        call    wait_64ms         	; wait a "long" time,
        bcf     LCD_e       		; and then Clear E 
        movlw   0x03              	; LCD init instruction (2nd)RB:0-3
        movwf   PORTB             	; Send to LCD via RB3..RB0
        bsf     LCD_e       		; Set E high,
        call    wait_32ms         	; wait a while,
        bcf     LCD_e       		; and then Clear E 
        movlw   0x03              	; LCD init instruction (Third)
        movwf   PORTB             	; Send to LCD via RB3..RB0
        bsf     LCD_e       		; Set E high,
        call    wait_32ms         	; wait a while,
        bcf     LCD_e       		; and then Clear E
        movlw   0x02              	; 4-bit mode instruction
        movwf   PORTB             	; Send to LCD via RB7..RB0
        bsf     LCD_e       		; Set E high,
        call    wait_16ms         	; wait a while,
        bcf     LCD_e       		; and then Clear E
;
        movlw   0x28              	; 1/16 duty cycle, 5x8 matrix
        call    cmnd2LCD          	; Send command in w to LCD
        movlw   0x08              	; Display off, cursor and blink off 
        call    cmnd2LCD          	; Send command to LCD
        movlw   0x01              	; Clear and reset cursor
        call    cmnd2LCD          	; Send command in w to LCD
        movlw   0x06              	; Set cursor to move right, no shift
        call    cmnd2LCD          	; Send command in w to LCD
        movlw   0x0C              	; Display on, cursor and blink off
        call    cmnd2LCD          	; Send command in w to LCD
        return                    	; 
;
add_step
        movf    fstep_0,w         	; Get low byte of the increment
        addwf   freq_0,f          	; Add it to the low byte of freq
        btfss   STATUS,C        	; Any carry?
        goto    add1              	; No, add next byte
        incfsz  freq_1,f          	; Ripple carry up to the next byte
        goto    add1              	; No new carry, add next byte
        incfsz  freq_2,f          	; Ripple carry up to the next byte
        goto    add1              	; No new carry, add next byte
        incf    freq_3,f          	; Ripple carry up to the highest byte
add1
        movf    fstep_1,w         	; Get the next increment byte
        addwf   freq_1,f          	; Add it to the next higher byte
        btfss   STATUS,C        	; Any carry?
        goto    add2              	; No, add next byte
        incfsz  freq_2,f          	; Ripple carry up to the next byte
        goto    add2              	; No new carry, add next byte
        incf    freq_3,f          	; Ripple carry up to the highest byte
add2
        movf    fstep_2,w         	; Get the next to most significant increment
        addwf   freq_2,f          	; Add it to the freq byte
        btfss   STATUS,C        	; Any carry?
        goto    add3              	; No, add last byte
        incf    freq_3,f          	; Ripple carry up to the highest byte
add3
        movf    fstep_3,w         	; Get the most significant increment byte
        addwf   freq_3,f          	; Add it to the most significant freq
        return                    	; Return to the caller
;
; * Purpose:  Check if freq exceeds the upper limit.                          *
;
check_add
;
;       Check the most significant byte.
;
        movlw   0xFF-limit_3      	; Get (FF - limit of high byte)
        addwf   freq_3,w          	; Add it to the current high byte
        btfsc   STATUS,C        	; Was high byte too large?
        goto    set_max           	; Yes, apply limit
        movlw   limit_3           	; Get high limit value
        subwf   freq_3,w          	; Subtract the limit value
        btfss   STATUS,C        	; Are we at the limit for the byte?
        goto    exit1             	; No, below.  Checks are done.
;
;       Check the second most significant byte.
;
        movlw   0xFF-limit_2      	; Get (FF - limit of next byte)
        addwf   freq_2,w          	; Add it to the current byte
        btfsc   STATUS,C        	; Is the current value too high?
        goto    set_max           	; Yes, apply the limit
        movlw   limit_2           	; Second limit byte
        subwf   freq_2,w          	; Subtract limit value
        btfss   STATUS,C        	; Are we at the limit for the byte?
        goto    exit1             	; No, below.  Checks are done.
;
;       Check the third most significant byte.
; 
        movlw   0xFF-limit_1      	; Get (FF - limit of next byte)
        addwf   freq_1,w          	; Add it to the current byte
        btfsc   STATUS,C        	; Is the current value too high?
        goto    set_max           	; Yes, apply the limit
        movlw   limit_1           	; Third limit byte
        subwf   freq_1,w          	; Subtract limit value
        btfss   STATUS,C        	; Are we at the limit for the byte?
        goto    exit1             	; No, below.  Checks are done.
; 
;       Check the least significant byte.
;
        movlw   limit_0           	; Fourth limit byte
        subwf   freq_0,w          	; Subtract limit value
        btfss   STATUS,C        	; Are we at the limit for the byte?
        goto    exit1             	; No, below.  Checks are done.
set_max
        movlw   limit_0           	; Get least significant limit
        movwf   freq_0            	; Set it in freq
        movlw   limit_1           	; Get the next byte limit
        movwf   freq_1            	; Set it in freq_1
        movlw   limit_2           	; Get the next byte limit
        movwf   freq_2            	; Set it in freq_2
        movlw   limit_3           	; Get the most significant limit
        movwf   freq_3            	; Set it in freq_3
exit1
        return                    	; Return to the caller
;
; * Purpose:  Subtract the increment step from freq, checking that it does    *
; *           not go below zero.                                              *
;
sub_step
        comf    fstep_0,f         	; Subtraction of fstep from
        comf    fstep_1,f         	; freq is done by adding the
        comf    fstep_2,f         	; twos compliment of fstep to
        comf    fstep_3,f         	; freq.
        incfsz  fstep_0,f         	; Increment last byte
        goto    comp_done         	; Non-zero, continue
        incfsz  fstep_1,f         	; Increment next byte
        goto    comp_done         	; Non-zero, continue
        incfsz  fstep_2,f         	; Increment next byte
        goto    comp_done         	; Non-zero, continue
        incf    fstep_3,f         	; Increment the high byte
comp_done
        call    add_step          	; Add the compliment to do the subtraction
;
;       If the frequency has gone negative, clear it to zero.
;
        btfss   freq_3,7          	; Is high order frequency byte "negative"?   
        goto    exit2             	; No, keep going
set_min
        clrf    freq_0            	; Yes, set the frequency to zero
        clrf    freq_1            	; 
        clrf    freq_2            	; 
        clrf    freq_3            	; 
exit2
        return                    	; Return to the caller
;
;------------------------------------------------------------
;
poll_encoder
        clrf    ren_timer_1       	; Put starting values in ren_timer
        movlw   0x40              	; Start with the high bit set in ren_timer
        movwf   ren_timer_0       	; 
read_encoder
        btfsc   ren_timer_1,7     	; Has the bit floated to bottom of ren_timer?
        goto    no_inc            	; Yes, don't move it any further
        movlw   0x08              	;
        addwf   ren_timer_0,f     	; 
	skpnc				; Did the add force a carry?
        incf    ren_timer_1,f     	; Yes, then add one to ren_timer_1
no_inc                            	 
        movfw   PORTA           	; Get the current encoder value
        movwf   ren_read          	; Save it
        movlw   0x03              	; Get encoder mask
        andwf   ren_read,w        	; Isolate encoder bits
        movwf   ren_new           	; Save new value
        xorwf   ren_old,w         	; Has it changed?
	skpnz
        goto    read_encoder      	; No, keep looking until it changes

;
;       Determine which direction the encoder turned.
;
	clrc				; Clear the carry bit
        rlf     ren_old,f         	; 
        movfw   ren_new         	; 
        xorwf   ren_old,f         	; 
        movfw   ren_old         	; 
        andlw   0x02              	; Mask only bit 2
        movwf   next_dir          	; Result to new dir
        xorwf   last_dir,w        	; Compare to last dir
;
;       Prevent encoder slip from giving a false change in direction.
;
;        btfsc   STATUS,Z        	; Zero?
	skpnz				; 
        goto    pe_continue       	; No slip; keep going
        movfw   next_dir        	; Yes, update direction
        movwf   last_dir          	; 
        movfw   ren_new         	; Save the current encoder bits for next time
        movwf   ren_old           	; 
        goto    read_encoder      	; Try again
pe_continue
        btfsc   ren_old,1         	; Are we going down?
        goto    up2               	; No, indicate we are going up
        clrf    last_dir          	; Yes, clear last_dir
        goto    exit3             	; Finish and return
up2 
        movlw   0x02              	; Set UP value in last_dir
        movwf   last_dir          	; 
exit3 
        movfw   ren_new         	; Get the current encoder bits
        movwf   ren_old           	; Save them in ren_old for the next time
;
	incf	morse_bits,F		; Only use every other change
	btfss	morse_bits,0		; /2
	goto	poll_encoder		; Loop back
        return                    	; Return to the caller
;
;------------------------------------------------------------
;
calibrate
        movlw   0x60              	; Set frequency to 10.140 MHz by
        movwf   freq_0            	; setting freq to the binary equivalent
        movlw   0xb9              	; of 10,140,000.
        movwf   freq_1            	; . 
        movlw   0x9a              	; . 
        movwf   freq_2            	; .
        movlw   0x00              	; .
        movwf   freq_3            	; .
;
; Read the starting reference oscillator value form EEPROM.
;
        clrf    ee_addr             	; Reset the EEPROM read address
        call    ee_read       		; Read EEPROM
        movf    ee_data,w          	; Get the first osc byte
        movwf   osc_0             	; Save osc frequency
	incf	ee_addr,F		; next address
        call    ee_read       		; Get next byte
        movf    ee_data,w          	; 
        movwf   osc_1             	; Save it
	incf	ee_addr,F		; next address
        call    ee_read       		; Get the third byte
        movf    ee_data,w          	; 
        movwf   osc_2             	; Save it
	incf	ee_addr,F		; next address
        call    ee_read       		; Get the fourth byte
        movf    ee_data,w          	;
        movwf   osc_3             	; Save it
        call    bin2BCD           	; Calculate BCD version of 10,000.00
        call    show_freq         	; Display the frequency on the LCD
        movlw   0x8C              	; Point LCD at digit 12
        movwf   LCD_char          	; 
        call    cmnd2LCD          	; 
        movlw   'C'               	; Send a C
        movwf   LCD_char          	; 
        call    data2LCD          	; 
        movlw   'A'               	; Send an A
        movwf   LCD_char          	; 
        call    data2LCD          	; 
        movlw   'L'               	; Send an L
        movwf   LCD_char          	; 
        call    data2LCD          	; 
;
        call    calc_dds_word     	; Calculate DDS value
        call    send_dds_word     	; Do 1st send twice to insure on
cal_loop
        call    calc_dds_word     	; Calculate DDS value based on current osc
        call    send_dds_word     	; Update the DDS chip
        call    poll_encoder      	; Wait until the encoder has moved.
        clrf    fstep_3           	; Clear the three most significant
        clrf    fstep_2           	; bytes of fstep
        clrf    fstep_1           	; 
        movlw   0x10              	; Assume that we are adjusting slowly
        movwf   fstep_0           	; Use small increment
        btfsc   ren_read,4        	; Was the encoder changing slowly?
        goto    update_osc        	; Yes, then continue with small increment
        movlw   0x80              	; No, then use the large increment
        movwf   fstep_0           	;
update_osc
        nop                       	; Wait a cycle
        btfsc   last_dir,1        	; Are we moving down?
        goto    faster            	; No, increase the osc value
;
;       slower
;
        comf    fstep_0,f         	; Subtraction of fstep is done by
        comf    fstep_1,f         	; adding the twos compliment of fsetp
        comf    fstep_2,f         	; to osc
        comf    fstep_3,f         	; 
        incfsz  fstep_0,f         	; Increment last byte
        goto    faster            	; Non-zero, continue
        incfsz  fstep_1,f         	; Increment next byte
        goto    faster            	; Non-zero, continue
        incfsz  fstep_2,f         	; Increment next byte
        goto    faster            	; Non-zero, continue
        incf    fstep_3,f         	; Increment the high byte
faster
        movf    fstep_0,w         	; Get the low byte increment
        addwf   osc_0,f           	; Add it to the low osc byte
        btfss   STATUS,C        	; Was there a carry?
        goto    add4              	; No, add the next bytes
        incfsz  osc_1,f           	; Ripple carry up to the next byte
        goto    add4              	; No new carry, add the next bytes
        incfsz  osc_2,f           	; Ripple carry up to the next byte
        goto    add4              	; No new carry, add the next bytes
        incf    osc_3,f           	; Ripple carry up to the highest byte
add4
        movf    fstep_1,w         	; Get the second byte increment
        addwf   osc_1,f           	; Add it to the second osc byte
        btfss   STATUS,C        	; Was there a carry?
        goto    add5              	; No, add the third bytes
        incfsz  osc_2,f           	; Ripple carry up to the next byte
        goto    add5              	; No new carry, add the third bytes
        incf    osc_3,f           	; Ripple carry up to the highest byte
add5
        movf    fstep_2,w         	; Get the third byte increment
        addwf   osc_2,f           	; Add it to the third osc byte
        btfss   STATUS,C        	; Was there a carry?
        goto    add6              	; No, add the fourth bytes
        incf    osc_3,f           	; Ripple carry up to the highest byte
add6
        movf    fstep_3,w         	; Get the fourth byte increment
        addwf   osc_3,f           	; Add it to the fourth byte
        btfss   ren_read,6        	; Is the button still pressed?
        goto    cal_loop          	; Yes, stay in calibrate mode
        clrf    ee_addr             	; Write final value to EEPROM 
        movf    osc_0,w           	; Record the first
        movwf   ee_data            	; osc
        call    ee_write      		; byte
	incf	ee_addr,f		; inc EE adr
        movf    osc_1,w           	; Record the second
        movwf   ee_data            	; osc
        call    ee_write      		; byte
	incf	ee_addr,f		;
        movf    osc_2,w           	; Record the third
        movwf   ee_data            	; osc
        call    ee_write      		; byte
	incf	ee_addr,f		;
        movf    osc_3,w           	; Record the fourth
        movwf   ee_data            	; osc
        call    ee_write      		; byte
        return                    	; Return to the caller
;
;------------------------------------------------------------
;
calc_dds_word
        clrf    AD9851_0          	; Clear the AD9851 control word bytes
        clrf    AD9851_1          	; 
        clrf    AD9851_2          	; 
        clrf    AD9851_3          	; 
        clrf    AD9851_4          	; 
        movlw   0x20              	; Set count  to 32   (4 osc bytes of 8 bits)
        movwf   mult_count        	; Keep running count
        movf    osc_0,w           	; Move the four osc bytes
        movwf   osc_temp_0        	; to temporary storage for this multiply
        movf    osc_1,w           	; (Don't disturb original osc bytes)
        movwf   osc_temp_1        	; 
        movf    osc_2,w           	; 
        movwf   osc_temp_2        	; 
        movf    osc_3,w           	; 
        movwf   osc_temp_3        	; 
mult_loop
	clrc				; Start with Carry clear
        btfss   osc_temp_0,0      	; Is bit 0 (Least Significant bit) set?
        goto    noAdd             	; No, don't need to add freq term to total
        movf    freq_0,w          	; Yes, get the freq_0 term
        addwf   AD9851_1,f        	; and add it in to total
	skpc				; Does this addition result in a carry?
        goto    add7              	; No, continue with next freq term
        incfsz  AD9851_2,f        	; Yes, add one and check for another carry
        goto    add7              	; No, continue with next freq term
        incfsz  AD9851_3,f        	; Yes, add one and check for another carry
        goto    add7              	; No, continue with next freq term
        incf    AD9851_4,f        	; Yes, add one and continue
add7
        movf    freq_1,w          	; Use the freq_1 term
        addwf   AD9851_2,f        	; Add freq term to total in correct position
	skpc				; Does this addition result in a carry?
        goto    add8              	; No, continue with next freq term
        incfsz  AD9851_3,f        	; Yes, add one and check for another carry
        goto    add8              	; No, continue with next freq term
        incf    AD9851_4,f        	; Yes, add one and continue
add8
        movf    freq_2,w          	; Use the freq_2 term
        addwf   AD9851_3,f        	; Add freq term to total in correct position
	skpc				; Does this addition result in a carry?
        goto    add9              	; No, continue with next freq term
        incf    AD9851_4,f        	; Yes, add one and continue
add9
        movf    freq_3,w          	; Use the freq_3 term
        addwf   AD9851_4,f        	; Add freq term to total in correct position
noAdd
        rrf     AD9851_4,f        	; Shift next multiplier bit into position
        rrf     AD9851_3,f        	; Rotate bits to right from byte to byte
        rrf     AD9851_2,f        	; 
        rrf     AD9851_1,f        	; 
        rrf     AD9851_0,f        	; 
        rrf     osc_temp_3,f      	; Shift next multiplicand bit into position
        rrf     osc_temp_2,f      	; Rotate bits to right from byte to byte
        rrf     osc_temp_1,f      	; 
        rrf     osc_temp_0,f      	; 
        decfsz  mult_count,f      	; One more bit has been done.  Are we done?
        goto    mult_loop         	; No, go back to use this bit
;
        movlw   0x01              	; Set the 6x reference oscilator bit
        movwf   AD9851_4	  	; 
        return                    	; Done.
;
;------------------------------------------------------------
;
send_dds_word
        movlw   AD9851_0          	; Point FSR at AD9851
        movwf   FSR               	; 
next_byte
        movf    INDF,w            	; 
        movwf   byte2send         	; 
        movlw   0x08              	; Set counter to 8
        movwf   bit_count         	; 
next_bit
        rrf     byte2send,f       	; Test if next bit is 1 or 0
	skpc				; Was it zero?
        goto    send0             	; Yes, send zero
        bsf     PORTB,3           	; No, send one
        bsf     PORTB,1           	; Toggle write clock 
        bcf     PORTB,1          	; 
        goto    break             	; 
send0
        bcf     PORTB,3           	; Send zero
        bsf     PORTB,1           	; Toggle write clock
        bcf     PORTB,1           	;
break
        decfsz  bit_count,f       	; Has the whole byte been sent?
        goto    next_bit          	; No, keep going.
        incf    FSR,f             	; Start the next byte unless finished
        movlw   AD9851_4+1        	; Next byte (past the end)
        subwf   FSR,w             	; 
	skpc
        goto    next_byte         	;

        bsf     PORTB,4           	; Send load signal to the AD9851
        bcf     PORTB,4           	; 
        return                    	;
;
bin2BCD
        movlw   0x20              	; Set loop counter
        movwf   BCD_count         	; to 32
        clrf    BCD_0             	; Clear output
        clrf    BCD_1             	;   "     "
        clrf    BCD_2             	;   "     "
        clrf    BCD_3             	;   "     "
        clrf    BCD_4             	;   "     "
bin_loop
	clrc				; Clear carry bit in STATUS
;
; Rotate bits in freq bytes.  Move from LS byte (freq_0) to next byte (freq_1).
; Likewise, move from freq_1 to freq_2 and from freq_2 to freq_3.
;
        rlf     freq_0,f          	; Rotate left, 0 -> LS bit, MS bit -> Carry
        rlf     freq_1,f          	; Rotate left, Carry->LS bit, MS bit->Carry
        rlf     freq_2,f          	; Rotate left, Carry->LS bit, MS bit->Carry
        rlf     freq_3,f          	; Rotate left, Carry->LS bit, MS bit->Carry
	skpnc				; Is Carry clear? If so, skip next instruction
        bsf     freq_0,0          	; Carry is set so wrap and set bit 0 in freq_0
;
; Build BCD bytes. Move into LS bit of BCD bytes (LS of BCD_0) from MS bit of
; freq_3 via the Carry bit.  
;
        rlf     BCD_0,f           	; Rotate left, Carry->LS bit, MS bit->Carry
        rlf     BCD_1,f           	; Rotate left, Carry->LS bit, MS bit->Carry
        rlf     BCD_2,f           	; Rotate left, Carry->LS bit, MS bit->Carry
        rlf     BCD_3,f           	; Rotate left, Carry->LS bit, MS bit->Carry
        rlf     BCD_4,f           	; Rotate left, Carry->LS bit, MS bit->Carry
        decfsz  BCD_count,f       	; Decrement loop count
        goto    adjust            	; No, go to adjust
        return                    	; Yes, EXIT 
; ============================================================================
adjust  ; Internal subroutine, called by bin2BCD main loop only
; 
; As BCD bytes are being built, make sure the nibbles do not grow larger than 9. 
; If a nibble gets larger than 9, increment to next higher nibble.  
; (If the LS nibble of a byte overflows, increment the MS nibble of that byte.)
; (If the MS nibble of a byte overflows, increment the LS nibble of next byte.)
;
        movlw   BCD_0             	; Get pointer to BCD_0
        movwf   FSR               	; Put pointer in FSR for indirect addressing
        call    adj_BCD           	; 
        incf    FSR,f             	; Move indirect addressing pointer to BCD_1
        call    adj_BCD           	; 
        incf    FSR,f             	; Move indirect addressing pointer to BCD_2
        call    adj_BCD           	; 
        incf    FSR,f             	; Move indirect addressing pointer to BCD_3
        call    adj_BCD           	; 
        incf    FSR,f             	; Move indirect addressing pointer to BCD_4
        call    adj_BCD           	; 
        goto    bin_loop          	; Back to main loop of bin2BCD
; ============================================================================
adj_BCD  ; Internal subroutine, called by adjust only
        movlw   3                 	; Add 3
        addwf   INDF,w            	;   to LS digit
        movwf   BCD_temp          	; Save in temp
        btfsc   BCD_temp,3        	; Is LS digit + 3 > 7  (Bit 3 set)
        movwf   INDF              	; Yes, save incremented value as LS digit
        movlw   0x30              	; Add 3
        addwf   INDF,w            	;   to MS digit
        movwf   BCD_temp          	; Save as temp
        btfsc   BCD_temp,7        	; Is MS digit + 3 > 7  (Bit 7 set)
        movwf   INDF              	; Yes, save incremented value as MS digit
        return                    	; Return to adjust subroutine
;
show_freq
        movlw   0x81              	; Point the LCD to first LCD digit location
        call    cmnd2LCD          	; Send starting digit location to LCD
;
; Running 4-bit mode, so need to send Most Significant Nibble first.
;
; Extract and send "XXXX" from byte containing "XXXXYYYY"
;  - Swap halves to get YYYYXXXX
;  - Mask with 0x0F to get 0000XXXX
;  - Add ASCII bias (0030XXXX)
;
        swapf   BCD_3,w           	; Swap 10MHz BCD digit into lower nibble of W
        andlw   0x0F              	; Mask for lower nibble only       (0000XXXX)
        addlw   0x30              	; Add offset for ASCII char set    (0030XXXX)
        call    data2LCD          	; Send byte in W to LCD
;
; Extract and send "YYYY" from byte containing "XXXXYYYY"
;   - Mask with 0x0F to get 0000YYYY
;   - Add offset for ASCII character set in LCD  (0030YYYY)
;
        movf    BCD_3,w           	; Put 1MHz BCD digit into lower nibble of W
        andlw   0x0F              	; Mask for lower nibble only       (0000YYYY)
        addlw   0x30              	; Add offset for ASCII char set    (0030YYYY)
        call    data2LCD          	; Send byte in W to LCD
; 
        movlw   ','               	; Get a comma
        call    data2LCD          	; Send byte in W to LCD
;
        swapf   BCD_2,w           	; Swap 100KHz BCD digit into lower nibble of W
        andlw   0x0F              	; Mask for lower nibble only       (0000XXXX)
        addlw   0x30              	; Add offset for ASCII char set    (0030XXXX)
        call    data2LCD          	; Send byte in W to LCD
;
        movf    BCD_2,w           	; Put 10KHz BCD digit into lower nibble of W
        andlw   0x0F              	; Mask for lower nibble only       (0000YYYY)
        addlw   0x30              	; Add offset for ASCII char set    (0030YYYY)
        call    data2LCD          	; Send byte in W to LCD
;
        swapf   BCD_1,w           	; Swap 1KHz BCD digit into lower nibble of W
        andlw   0x0F              	; Mask for lower nibble only       (0000XXXX)
        addlw   0x30              	; Add offset for ASCII char set    (0030XXXX)
        call    data2LCD          	; Send byte in W to LCD
;
        movlw   '.'               	; Set up W with ASCII Period 
        call    data2LCD          	; Send data byte in W to LCD
;
;
        movf    BCD_1,w           	; Put 100 Hz BCD digit into lower nibble of W
        andlw   0x0F              	; Mask for lower nibble only       (0000YYYY)
        addlw   0x30              	; Add offset for ASCII char set    (0030YYYY)
        call    data2LCD          	; Send data byte in W to LCD
;
        swapf   BCD_0,w           	; Swap 10 Hz BCD digit into lower nibble of W
        andlw   0x0F              	; Mask for lower nibble only       (0000XXXX)
        addlw   0x30              	; Add offset for ASCII char set    (0030XXXX)
        call    data2LCD          	; Send data byte in W to LCD
;
        movf    BCD_0,w           	; Put 1 Hz BCD digit into lower nibble of W
        andlw   0x0F              	; Mask for lower nibble only       (0000YYYY)
        addlw   0x30              	; Add offset for ASCII char set    (0030YYYY)
        call    data2LCD          	; Send byte in W to LCD
;
        movlw   ' '               	; Send a space
        call    data2LCD          	;   to LCD
;
        movlw   'k'               	; Send a 'k'
        call    data2LCD          	;   to LCD
;
        movlw   'H'               	; Send an "H"
        call    data2LCD          	;   to LCD
;
        movlw   'z'               	; Send a 'z'
        call    data2LCD          	;   to LCD
;
        return                    	; 
;
; *****************************************************************************
; *    		Modified for QRSS layout 5/2/11                                                                       *
; 
busy_check
        clrf    PORTB             	; Clear all outputs on PORTB
        bsf     STATUS,RP0      	; Switch to bank 1 for Tristate operation
        movlw   b'11001111'       	; Set RB3..RB0 as inputs, RB5..RB4 outputs
        movwf   TRISB             	; via Tristate
        bcf     STATUS,RP0      	; Switch back to bank 0
        bcf     LCD_rs      		; Set up LCD for Read Busy Flag (RS = 0) 
        bsf     LCD_rw      		; Set up LCD for Read (RW = 1)  
        movlw   0xFF              	; Set up constant 255
        movwf   timer1            	; for timer loop counter
LCD_is_busy
        bsf     LCD_e       		; Set E high
        movf    PORTB,w           	; Read PORTB into W
        movwf   LCD_read          	; Save W for later testing
        bcf     LCD_e       		; Drop E again
        nop                       	; Wait a while
        nop                       	;   
        bsf     LCD_e       		; Pulse E high (dummy read of lower nibble),
        nop                       	; wait,
        bcf     LCD_e       		; and drop E again
        decf    timer1,f          	; Decrement loop counter
        btfsc   STATUS,Z        	; Is loop counter down to zero?
        goto    not_busy          	; If yes, return regardless
        btfsc   LCD_read,3        	; Is Busy Flag (RB7) in save byte clear?
        goto    LCD_is_busy       	; If not, it is busy so jump back
not_busy
        return                    	; 
;
cmnd2LCD   ; ****** Entry point ******
        movwf   LCD_char          	; Save byte to write to LCD
        clrf    rs_value          	; Remember to clear RS  (clear rs_value)   
        bcf     LCD_rs      		; Set RS for Command to LCD
        goto    write2LCD         	; Go to common code
data2LCD   ; ****** Entry point ********
        movwf   LCD_char          	; Save byte to write to LCD
        bsf     rs_value,0        	; Remember to set RS (set bit 0 of rs_value)
        bsf     LCD_rs      		; Set RS for Data to LCD
;
write2LCD
        call    busy_check        	; Check to see if LCD is ready for new data
        clrf    PORTB             	; Clear all of Port B (inputs and outputs)
        bsf     STATUS,RP0      	; Switch to bank 1 for Tristate operation
        movlw   b'11000000'      	; Set up to enable PORTB data pins<<<
        movwf   TRISB             	; Pins (RB5..RB0) are back to outputs
        bcf     STATUS,RP0      	; Switch to bank 0
        bcf     LCD_rw      		; Set LCD back to Write mode  (RW = 0)
        bcf     LCD_rs      		; Guess RS should be clear              
        btfsc   rs_value,0        	; Should RS be clear?  (is bit 0 == 0?) 
        bsf     LCD_rs      		; No, set RS                            
;
; Transfer Most Significant nibble  (XXXX portion of XXXXYYYY)
;
        movlw   0xF0              	; Set up mask                                 
        andwf   PORTB,f           	; Clear old RB3..RB0 
        swapf   LCD_char,w        	; Move MS nibble of data to LS position in W
        andlw   0x0F              	; Mask to give 0000XXXX in W
        iorwf   PORTB,f           	; Send to RB3..RB0 without changing RB7..RB4
        bsf     LCD_e       		; Pulse the E line high,
        nop                       	;   wait, 
        bcf     LCD_e       		; and drop it again
;
; Transfer Least Significant nibble  (YYYY portion of XXXXYYYY)
;
        movlw   0xF0              	; Set up mask                                 
        andwf   PORTB,f           	; Clear old RB3..RB0
        movf    LCD_char,w        	; Put byte of data into W
        andlw   0x0F              	; Mask to give 0000YYYY in W
        iorwf   PORTB,F             	; Send to RB3..RB0 without changing RB7..RB4
        bsf     LCD_e       		; Pulse the E line high,
        nop                       	; wait, 
        bcf     LCD_e       		; and drop it again
        return
;
line1_clearer
	movlw	.16			; num chars to erase
	movwf	count1			;
	movlw	LINE1
        movwf   LCD_char          	; 0x80 to LCD
        call    cmnd2LCD          	; 
line1_clearer_loop
	movlw	" "			; space
        movwf   LCD_char          	; W to LCD
        call    data2LCD          	; display
	decfsz	count1,F		;
	goto	line1_clearer_loop		;
	return
;
line2_clearer
	movlw	.16			; num chars to erase
	movwf	count1			;
	movlw	LINE2
        movwf   LCD_char          	; 0xC0 to LCD
        call    cmnd2LCD          	; 
line2_clearer_loop
	movlw	" "			; space
        movwf   LCD_char          	; W to LCD
        call    data2LCD          	; display
	decfsz	count1,F		;
	goto	line2_clearer_loop	;
	return
;
;------------------------------------------------------------
ee_write
	movlw	EEADR			;setup for EE write
	movwf	FSR			;
	movfw	ee_addr			;address 1st
	movwf	INDF			;
	movlw	EEDATA			;
	movwf	FSR			;
	movfw	ee_data			;then data
	movwf	INDF			;
;
	bsf	STATUS,RP0		;interrupts off between here 
	bsf	EECON1,WREN		;
	movlw	0x55
	movwf	EECON2
	movlw	0xaa
	movwf	EECON2
	bsf	EECON1,WR
ee_wr_wait
	btfsc	EECON1,WR
	goto	ee_wr_wait
	bcf	EECON1,WREN
	bcf	STATUS,RP0		;and here
	return
;
ee_read
	movlw	EEADR			;setup for EE read
	movwf	FSR
	movfw	ee_addr			;address
	movwf	INDF
	bsf	STATUS,RP0
	bsf	EECON1,RD
	bcf	STATUS,RP0
	movlw	EEDATA
	movwf	FSR
	movfw	INDF
	movwf	ee_data			;data
	return
;
;
wait_128ms  ; ****** Entry point ******    
        movlw   0xFF              	; Set up outer loop 
        movwf   timer1            	;   counter to 255
        goto    outer_loop        	; Go to wait loops
wait_64ms  ; ****** Entry point ******     
        movlw   0x80              	; Set up outer loop
        movwf   timer1            	;   counter to 128

        goto    outer_loop        	; Go to wait loops
wait_32ms   ; ****** Entry point ******    
        movlw   0x40              	; Set up outer loop
        movwf   timer1            	;   counter to 64
        goto    outer_loop        	; Go to wait loops
wait_16ms   ; ****** Entry point ******    
        movlw   0x20              	; Set up outer loop
        movwf   timer1            	;   counter to 32  
        goto    outer_loop        	; Go to wait loops
wait_8ms   ; ****** Entry point ******     
        movlw   0x10              	; Set up outer loop
        movwf   timer1            	;   counter to 16
                                  	; Fall through into wait loops
;
; Wait loops used by other wait routines
;  - 1 microsecond per instruction (with a 4 MHz microprocessor crystal)
;  - 510 instructions per inner loop
;  - (Timer1 * 514) instructions (.514 msec) per outer loop
;  - Round off to .5 ms per outer loop
;
outer_loop                        
        movlw   0xFF              	; Set up inner loop counter
        movwf   timer2            	;   to 255
inner_loop
        decfsz  timer2,f          	; Decrement inner loop counter
        goto    inner_loop        	; If inner loop counter not down to zero, 
                                  	; then go back to inner loop again
        decfsz  timer1,f          	; Yes, Decrement outer loop counter
        goto    outer_loop        	; If outer loop counter not down to zero,
                                  	; then go back to outer loop again
        return                    	; Yes, return to caller
;
;------------------------------------------------------------
;
;
 	ORG	0x400
raw_morse
	addwf	PCL,F
;
	dt	0x68,0xe7,0xee,0xe1,0x18,0x7e,0xf1,0x77,0x78		;A-I
;
	dt	0x6f,0xe9,0x67,0xf8,0xe8,0xf9,0x6e,0xf6,0x61		;J-R
;
	dt	0x71,0x98,0x79,0x76,0x69,0xe6,0xef,0xf7			;S-Z
;	
raw_numbers
	addwf	PCL,F
	dt	0xd8,0x1c,0x2b,0x3a,0x49,0x58,0x94,0xf3,0xb7,0xc1	;0-9
;
;------------------------------------------------------------
;	Rebuilt 5/2011 N0QBH
;	This part converts ASCII from EEmem into a continous 
;	serial bit stream w/ 6 sec bits
;
bin_builder
	movlw	.8
	movwf	pointer1		;points to ee mem loc
	movwf	ee_addr			;set ee address
	clrf	pass_cnt		;reset pass counter
	clrf	morse_bits		;morse string length (<256)	
	movlw	RAM_START			;
	movwf	pointer			;start of binary storage
	movlw	.8
	movwf 	roll_cnt		;preload roll cnt
	movlw	high raw_morse
	movwf	PCLATH			;set up PCLATH
;------------------------------------------------------------		
;	Numbers		30h-39h		0-9
;	Uppercase	41h-5Ah		A-Z
;	Lowercase	61h-7Ah		a-z
;	Also does space (0x20h)
;------------------------------------------------------------
;	Limited to 15 chars
;
read_ascii_text
	movfw	pointer1		;load pointer1 to W
	movwf	ee_addr			;W to EE adr
;
	call	ee_read			;read EE mem
	movfw	ee_data
	movwf	buffer_1		;store returned byte
	movfw	buffer_1		;test for zero
	skpnz	
	goto	sentence_finished	;returned zero, done
;
	sublw	0x7a			;7Ah - W
	skpc				;ignore above "z"
	goto	bin_builder		;>7a, out of range
	movlw	0x61			;W = 61h
	subwf	buffer_1,W		;
	bc	lcase			;=> 0x61, in lower case range
;
	movfw	buffer_1		;ignore 5Bh to 60h
	sublw	0x5a			;5Ah - W
	skpc	
	goto	bin_builder		;>5a, out of range	
	movlw	0x41			;
	subwf	buffer_1,W		;41h - W
	bc	ucase			;=> 41h, in upper case range
;
	movfw	buffer_1		;ignore 3Ah to 40h
	sublw	0x39			;39h - W
	skpc
	goto	bin_builder		;>39, out of range
	movlw	0x30
	subwf	buffer_1,W		;buffer1 - 30h
	bc	numerals		;=> 30h
;
	movlw	0x20			;test for space
	xorwf	buffer_1,W
	skpz
	goto	bin_builder		;out of range
	goto	space_place		;ASCII space - we send dash low
;
lcase	
	movlw	0x61
	subwf	buffer_1,W		;0-25 result
	call	raw_morse
	movwf	rm_buffer		;store returned W
	goto	decode_nibble
;
ucase
	movlw	0x41
	subwf	buffer_1,W		;0-25 result
	call	raw_morse
	movwf	rm_buffer		;store returned W
	goto	decode_nibble
;
numerals
	movlw	0x30
	subwf	buffer_1,W		;0-9 result
	andlw	0x0f			;
	call	raw_numbers		;get from table
	movwf	rm_buffer		;store returned W	
;
decode_nibble
	swapf	rm_buffer,F		;send upper nibble 1st
	movfw	rm_buffer		;
	andlw	0x0f			;mask unused
	addwf	PCL,F			;low nibble becomes pointer
	goto	rollcallend		;W=0 unused, skip to end
	goto	dot_1			;W=1
	goto	dot_2			;W=2
	goto	dot_3			;
	goto	dot_4			;
	goto	dot_5			;			
	goto	dit_dah			;
	goto	dot_2			;
	goto	rollcallend		;W=8 skip to end
	goto	dash_1			;		
	goto	dash_2			;
	goto	dash_3			;
	goto	dash_4			;
	goto	dash_5			;			
	goto	dah_dit			;
	goto	dash_2			;W=15
;
sentence_finished
	movlw	.8
	xorwf	roll_cnt,W		;test for 8
	bnz	wind_up			;not 8
	decf	pointer,F		;correct pointer
	goto	bb_cleanup		;next not needed
wind_up	
	clrc
	rlf	buffer_2,F		;left shift
	incf	morse_bits,F		;count em
	decfsz	roll_cnt,F		;
	goto	wind_up			;roll_cnt is 0
	movfw	pointer			;point to last 	
	movwf	FSR			;
	movfw	buffer_2		;now properly left shifted
	movwf	INDF			;store
bb_cleanup
	clrf	freq_0			;reset these regs
	clrf	freq_1
	clrf	freq_2
	clrf	freq_3
	return				;leave bin build
;
;
dot_1
	movlw	.2
	movwf	freq_3			;2 bits long
	movlw	b'00000001'		;
	movwf	freq_0			;
	clrf	freq_1			;
	clrf	freq_2			;
	call	rollcall		;
	goto	rollcallend		;
;
dot_2
	movlw	.4
	movwf	freq_3			;
	movlw	b'00000101'		;4 bits long
	movwf	freq_0			;
	clrf	freq_1			;
	clrf	freq_2			;
	call	rollcall		;
	goto	rollcallend		;
;
dot_3
	movlw	.6
	movwf	freq_3			;6 bits long
	movlw	b'00010101'		;	
	movwf	freq_0			;
	clrf	freq_1			;
	clrf	freq_2			;
	call	rollcall		;
	goto	rollcallend		;
;
dot_4
	movlw	.8
	movwf	freq_3			;8 bits long
	movlw	b'01010101'		;
	movwf	freq_0			;
	clrf	freq_1			;
	clrf	freq_2			;
	call	rollcall		;
	goto	rollcallend		;
;
dot_5
	movlw	.10
	movwf	freq_3			;10 bits long
	movlw	b'01010101'		;		
	movwf	freq_0			;
	movlw	b'00000001'		;
	movwf	freq_1			;
	clrf	freq_2			;
	call	rollcall		;
	goto	rollcallend		;
;
dit_dah
	movlw	.6
	movwf	freq_3			;6 bits long
	movlw	b'00011101'		;
	movwf	freq_0			;
	clrf	freq_1			;
	clrf	freq_2			;
	call	rollcall		;
	goto	rollcallend		;
;
dash_1
	movlw	.4
	movwf	freq_3			;4 bits long
	movlw	b'00000111'		;
	movwf	freq_0			;
	clrf	freq_1			;
	clrf	freq_2			;
	call	rollcall		;
	goto	rollcallend		;
;
dash_2
	movlw	.8
	movwf	freq_3			;8 bits long
	movlw	b'01110111'		;
	movwf	freq_0			;
	clrf	freq_1			;
	clrf	freq_2			;
	call	rollcall		;
	goto	rollcallend		;
;
dash_3
	movlw	.12
	movwf	freq_3			;12 bits long
	movlw	b'01110111'		;
	movwf	freq_0			;
	movlw	b'00000111'		;
	movwf	freq_1			;
	clrf	freq_2			;
	call	rollcall		;
	goto	rollcallend		;
;
dash_4
	movlw	.16
	movwf	freq_3			;16 bits long
	movlw	b'01110111'		;
	movwf	freq_0			;
	movlw	b'01110111'		;
	movwf	freq_1			;
	clrf	freq_2			;
	call	rollcall		;
	goto	rollcallend		;
;
dash_5
	movlw	.20
	movwf	freq_3			;20 bits long
	movlw	b'01110111'		;
	movwf	freq_0			;
	movlw	b'01110111'		;
	movwf	freq_1			;
	movlw	b'00000111'		;
	movwf	freq_2			;
	call	rollcall		;
	goto	rollcallend		;
;
dah_dit
	movlw	.6
	movwf	freq_3			;6 bits long
	movlw	b'00010111'		;
	movwf	freq_0			;
	clrf	freq_1			;
	clrf	freq_2			;
	call	rollcall		;
	goto	rollcallend		;
;
rollcallend
	incf	pass_cnt,F		;
	movlw	0x02			;=2?
	xorwf	pass_cnt,W		;
	skpz				;yes, skip next
	goto	decode_nibble		;do 2nd pass
	clrf	pass_cnt		;reset
	goto	twothirds		;between char space
;
;------------------------------------------------------------
;
space_place
	movlw	.3			;3 bits clear
	movwf	freq_3			;
	clrf	freq_0
	clrf	freq_1
	clrf	freq_2
	call	rollcall		;1 dash low tone
	incf	pointer1,F		;inc pointer1
	goto	read_ascii_text		;loop back for next
;
twothirds
	movlw	.2			;2 bits clear
	movwf	freq_3			;
	clrf	freq_0
	clrf	freq_1
	clrf	freq_2
	call	rollcall		;1 dash low tone
	incf	pointer1,F		;inc pointer1
	goto	read_ascii_text		;loop back for next
;
;------------------------------------------------------------
;
;	freq_3 contains number of bits
;	freq_0,1,2 contain bits	
;	Uses buffer_2, pointer, roll_cnt to create Morse string
;
rollcall
	clrc
	rrf	freq_2,F		;right shift one bit
	rrf	freq_1,F		;
	rrf	freq_0,F		;
;
	rlf	buffer_2,F		;left shift carry bit in
	decfsz	roll_cnt,F		;
	goto	rc_end			;not = 8, scram
	movlw	.8			;byte full, do next
	movwf	roll_cnt
	movfw	pointer			;point to last 	
	movwf	FSR			;
	movfw	buffer_2		;
	movwf	INDF			;store
	incf	pointer,W
	xorlw	0x7f
	skpz				;hold at mem end
	incf	pointer,F		;
;	
rc_end
	decfsz	freq_3,F
	goto	rollcall
	return
;
;------------------------------------------------------------
;==============================================================
;------------------------------------------------------------
; These variables are only used by the ascii to Morse and 
; cs_rodeo routines.	
;
;	count1				;
;	pass_cnt			;0-7 LCD position
;	morse_bits			;
;	buffer_1			;
;	rm_buffer			;
;	pointer1			;
;
;
;
	ORG	0x600
cs_rodeo_lookup
	addwf	PCL,F
	dt	"0123456789"
	dt	"ABCDEFGH"
	dt	"IJKLMNOP"
	dt	"QRSTUVWXYZ*" 
;
line1_message
	addwf	PCL,F
	dt	"Enter callsign",0
;
cs_rodeo
	clrf	count1			; clear these regs
	clrf	pass_cnt		;
	clrf	morse_bits		;
	clrf	buffer_1		;
	clrf	rm_buffer		;
	clrf	pointer1		; main pointer
;
	call	line2_clearer		; clear line 2
;
	movlw	RAM_START
	movwf	count1			; Temp store start
;
rodeo_diplay_setup
        movfw   PORTA            	; Get the current encoder value
        movwf   ren_read          	; Save it
	movlw	b'00010000'		; check only bit 4
        andwf   ren_read,w        	;
	skpnz				; bit 4 high?
	goto	rodeo_diplay_setup	; no, wait till open	
	movlw	high cs_rodeo_lookup	;
	movwf	PCLATH			; set high address bits
;
	movlw	LINE1
        movwf   LCD_char          	; display text 
        call    cmnd2LCD          	; beginning line1
rds_loop
	movfw	pass_cnt		; pass_count to W
	call	line1_message		; W is pointer
	movwf	LCD_char		; 
	movfw	LCD_char		; test for end zero
	bz	rds_loop_bail		; exit
        call    data2LCD          	; display text string
	incf	pass_cnt,F		;	
	goto	rds_loop
rds_loop_bail
	movlw	LINE2 +1
	movwf 	pass_cnt		; 0xC0-0xC7
;
;	Nitty Gritty single speed version
;	Uses ren_read, ren_new, ren_old, next_dir, last_dir
;
query_encoder
        movfw   PORTA            	; Get the current encoder value
        movwf   ren_read          	; Save it
	movlw	b'00010000'		; bit 4
        andwf   ren_read,w        	;
	bz	show_and_tell
        movlw   0x03              	; Get encoder mask
        andwf   ren_read,w        	; Isolate encoder bits
        movwf   ren_new           	; Save new value
        xorwf   ren_old,w         	; Has it changed?
       	skpnz        		  	; 
        goto    query_encoder     	; No, keep looking until it changes
;
;       Determine which direction the encoder turned.
;
        clrc		        	; Clear the carry bit
        rlf     ren_old,f         	; 
        movfw   ren_new         	; 
        xorwf   ren_old,f         	; 
        movfw   ren_old         	; 
        andlw   0x02              	; 
        movwf   next_dir          	; 
        xorwf   last_dir,w        	; 
;
;       Prevent encoder slip from giving a false change in direction.
;
        skpnz		          	; Zero?
        goto    query_continue       	; No slip; keep going
        movfw   next_dir         	; Yes, update direction
        movwf   last_dir          	; 
        movfw   ren_new          	; Save the current encoder bits for next time
        movwf   ren_old           	; 
        goto    query_encoder      	; Try again
query_continue
        btfsc   ren_old,1         	; Are we going down?
        goto    increased              	; No, indicate we are going up
        clrf    last_dir          	; Yes, clear last_dir
        goto    bail_enc             	; Finish and return
increased
        movlw   0x02              	; Set UP value in last_dir
        movwf   last_dir          	; 
bail_enc 
        movfw   ren_new         	; Get the current encoder bits
        movwf   ren_old           	; Save them in ren_old for the next time
	incf	morse_bits,F		;
	btfss	morse_bits,0		; /2
	goto	query_encoder		; skip next
;
        btfsc   last_dir,1        	; Is the knob going up?
        goto    up_alpha              	; Yes, then increment
down_alpha				; No, decrement
	decf	pointer1,W
	xorlw	.255			; hold at zero ("0")
	bz	wrap_down		; 
	decf	pointer1,F		; pointer1 -1
	goto	show_and_tell
;
wrap_down	
	movlw	.37			; 1 extra
	movwf	pointer1		;
	goto	down_alpha		;		
;
up_alpha
	incf	pointer1,W
	xorlw	.37			; hold at "*" (quit)
	bz	wrap_up			;
	incf	pointer1,F		; pointer1 +1
	goto	show_and_tell
;
wrap_up
	movlw	.255			; 
	movwf	pointer1		;
	goto	up_alpha		;
;
show_and_tell
	movfw	pass_cnt		; lcd char position
        movwf   LCD_char          	; 
        call    cmnd2LCD          	; points to which segment	
	movfw	pointer1		; points to letter /digit
	call	cs_rodeo_lookup		;
        movwf   LCD_char          	; 
	movwf	rm_buffer		; store copy of W
        call    data2LCD          	; display 1st char
;
	btfsc	ren_read,4		; tuner sw pressed?
	goto	query_encoder		; no
	call	wait_128ms		; yes, debounce
	movfw	PORTA
	movwf	ren_read		;		
	btfsc	ren_read,4		; tuner sw still pressed?
	goto	query_encoder		; no
	movfw	rm_buffer		; yes, test for "*"
	xorlw	"*"			; 	
	bz	show_stored_cs		; exit w/o store
;
good2store
	movfw	count1			;	
	movwf	FSR			; where to store selected char
	movfw	rm_buffer
	movwf	INDF			; store at offset
	incf	pass_cnt,W
	xorlw	LINE2 +.8		;
	skpz				; hold	
	incf	pass_cnt,F		; right 1 char
	incf	count1,W		;
	xorlw	RAM_START +.7		;
	skpz				; hold
	incf	count1,F		; next store loc
;
waitforgrampa	
	movfw	PORTA
	movwf	ren_read		;		
	btfss	ren_read,4		; tuner sw open?
	goto	waitforgrampa
	call	wait_128ms		; debounce
	movfw	PORTA
	movwf	ren_read		;		
	btfss	ren_read,4		; tuner sw still open?
	goto	waitforgrampa
	goto	query_encoder
;
show_stored_cs
	movlw	RAM_START		; start of user data
	movwf	buffer_1		; to buffer_1
	xorwf	count1,W		; test for no data
	bz	cs_rodeo		; no stored data, start over
	movlw	.8			; stored data found
	movwf	ee_addr			; start of call in ee
ss_cs_loop
	movfw	buffer_1
	movwf	FSR
	movfw	INDF			; get stored data
	movwf	ee_data
	call	ee_write		; store in eemem	
	incf	buffer_1,W		;
	xorwf	count1,W		; end of user data?	
	bz	indicate_stored		; yes, indicate with star
	incf	buffer_1,F		;
	incf	ee_addr,F		;
	goto	ss_cs_loop		;
;
indicate_stored
	incf	ee_addr,F		; 	
	clrf	ee_data			; send 0
	call	ee_write		; marks end of data
;
	clrf	count1			; clear these regs
	clrf	pass_cnt		;
	clrf	morse_bits		;
	clrf	buffer_1		;
	clrf	rm_buffer		;
	clrf	pointer1		;	
	return	
;
;============================================================
;
        ORG     0x2100
        DATA    ref_osc_0
        DATA    ref_osc_1
        DATA    ref_osc_2
        DATA    ref_osc_3
;
;       Clear unused EEPROM bytes.
;
	DATA	0,0,0,0
        de	"NOTCFG",0
;
;       
; *****************************************************************************
;
        END
